import React, { useEffect, useState } from 'react';
import { Table } from 'react-bootstrap';

const AdminViewOrders = () => {
    const [allUserOrders, setAllUserOrders] = useState([]);
    const [productNameList, setProductNameList] = useState([]);
    const [loadingOrders, setLoadingOrders] = useState(true); // State variable for loading orders
    const [loadingProductNames, setLoadingProductNames] = useState(true); // State variable for loading product names

    useEffect(() => {
        const fetchAllUserOrders = async () => {
            try {
                const response = await fetch(`${process.env.REACT_APP_API_BASE_URL}/users/get-all-orders`, {
                    headers: {
                        'Content-Type': 'application/json',
                        'Authorization': `Bearer ${localStorage.token}`,
                    },
                });

                if (response.ok) {
                    const ordersData = await response.json();
                    const updatedOrdersData = await Promise.all(
                        ordersData.map(async (user) => {
                            const userName = await fetchUserNames(user.userId);
                            return { ...user, userName };
                        })
                    );

                    setAllUserOrders(updatedOrdersData);
                } else {
                    const errorData = await response.json();
                    console.error('Error retrieving all user orders:', errorData.message);
                }
            } catch (error) {
                console.error('Error retrieving all user orders:', error);
            } finally {
                setLoadingOrders(false); // Set loading orders to false when fetching is complete
            }
        };

        fetchAllUserOrders();
    }, []);

    useEffect(() => {
        const fetchProductNames = async () => {
            setLoadingProductNames(true); // Set loading product names to true before fetching
            const names = [];
            for (const order of allUserOrders) {
                for (const product of order.products) {
                    try {
                        const response = await fetch(`${process.env.REACT_APP_API_BASE_URL}/products/getProductName`, {
                            method: 'POST',
                            headers: {
                                'Content-Type': 'application/json',
                            },
                            body: JSON.stringify({ productId: product.productId }),
                        });
                        if (response.ok) {
                            const data = await response.json();
                            names.push({
                                name: data.name,
                                productId: product.productId
                            });
                        } else {
                            throw new Error('Error retrieving product name');
                        }
                    } catch (error) {
                        console.error('Error retrieving product name:', error.message);
                        names.push(null);
                    }
                }
            }
            setProductNameList(names);
            setLoadingProductNames(false); // Set loading product names to false when fetching is complete
        };

        if (allUserOrders.length > 0) {
            fetchProductNames();
        }
    }, [allUserOrders]);

    const fetchUserNames = async (userId) => {
        try {
            const response = await fetch(`${process.env.REACT_APP_API_BASE_URL}/users/getUserName`, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                },
                body: JSON.stringify({ userId: userId }),
            });
            if (response.ok) {
                const data = await response.json();
                const userName = (`${data.firstName} ${data.lastName}`)
                return userName;
            } else {
                throw new Error('Error retrieving user name');
            }
        } catch (error) {
            console.error('Error retrieving user name:', error.message);
            return null;
        }
    };

    const userOrdersMap = allUserOrders.reduce((acc, order) => {
        if (!acc[order.userName]) {
            acc[order.userName] = [];
        }
        acc[order.userName].push(order);
        return acc;
    }, {});

    const UsersHistoryList = Object.entries(userOrdersMap).map(([user, orders]) => (
        <div key={user} className="container-fluid">
            <h3>User: {user}</h3>
            <Table striped bordered hover responsive className="h5 text-center">
                <thead className="text-center">
                    <tr>
                        <th className="h3">Purchase Date</th>
                        <th className="h3">Products</th>
                        <th className="h3">Order Quantity</th>
                        <th className="h3">Price</th>
                        <th className="h3">Total Amount</th>
                    </tr>
                </thead>
                <tbody>
                    {orders.map((order, index) => (
                        <tr key={index}>
                            <td>{order.purchasedOn}</td>
                            <td>
                                <ul style={{ listStyleType: 'none', padding: 0 }}>
                                    {loadingProductNames ? (
                                        <li>Loading product names...</li>
                                    ) : (
                                            order.products.map((product, productIndex) => {
                                                const productName = productNameList.find(item => item.productId === product.productId);
                                                return (
                                                    <li key={productIndex} className="text-start">
                                                        {productName ? productName.name : 'Product Name Not Found'}
                                                    </li>
                                                );
                                            })
                                    )}
                                </ul>
                            </td>
                            <td>
                                <ul style={{ listStyleType: 'none', padding: 0 }}>
                                    {order.products.map((product, productIndex) => (
                                        <li key={productIndex}>{product.quantity}</li>
                                    ))}
                                </ul>
                            </td>
                            <td>
                                <ul style={{ listStyleType: 'none', padding: 0 }}>
                                    {order.products.map((product, productIndex) => (
                                        <li key={productIndex}>{product.price.toLocaleString('en-PH', { style: 'currency', currency: 'PHP' })}</li>
                                    ))}
                                </ul>
                            </td>
                            <td>{order.totalAmount.toLocaleString('en-PH', { style: 'currency', currency: 'PHP' })}</td>
                        </tr>
                    ))}
                </tbody>
            </Table>
        </div>
    ));

    return (
        <div className="container p-5 forms d-flex flex-column align-items-center">
            <h2 className="mb-5 headers">Admin View: All User Orders</h2>
            {loadingOrders ? (
                <p>Loading orders...</p>
            ) : (
                UsersHistoryList
            )}
        </div>
    );
};

export default AdminViewOrders;
