import React, { useEffect, useState } from 'react';
import { Table, Spinner } from 'react-bootstrap'; // Import Spinner component

const ViewUserOrders = () => {
  const [userOrders, setUserOrders] = useState([]);
  const [productNameList, setProductNameList] = useState([]);
  const [isLoading, setIsLoading] = useState(true); // State variable to track loading status

  useEffect(() => {
    const fetchUserOrders = async () => {
      try {
        const response = await fetch(`${process.env.REACT_APP_API_BASE_URL}/users/orders`, {
          headers: {
            'Content-Type': 'application/json',
            'Authorization': `Bearer ${localStorage.token}`,
          },
        });

        if (response.ok) {
          const ordersData = await response.json();
          setUserOrders(ordersData.orders);
        } else {
          const errorData = await response.json();
          console.error('Error retrieving user orders:', errorData.message);
        }
      } catch (error) {
        console.error('Error retrieving user orders:', error);
      }
    };

    fetchUserOrders();

  }, []);

  useEffect(() => {
    const fetchProductNames = async () => {
      const names = [];
      for (const order of userOrders) {
        const orderNames = []; // Array to store product names for this order
        for (const product of order.products) {
          try {
            const response = await fetch(`${process.env.REACT_APP_API_BASE_URL}/products/getProductName`, {
              method: 'POST',
              headers: {
                'Content-Type': 'application/json',
              },
              body: JSON.stringify({ productId: product.productId }),
            });
            if (response.ok) {
              const data = await response.json();
              orderNames.push(data.name); // Push product name to orderNames array
            } else {
              throw new Error('Error retrieving product name');
            }
          } catch (error) {
            console.error('Error retrieving product name:', error.message);
            orderNames.push(null);
          }
        }
        names.push(orderNames); // Push orderNames array to names array
      }
      setProductNameList(names);
      setIsLoading(false); // Set loading to false after fetching product names
    };


    if (userOrders.length > 0) {
      fetchProductNames();
    }
  }, [userOrders]);

  const OrderList = userOrders.map((order, index) => {
    const { purchasedOn, totalAmount } = order;
    const originalDate = new Date(purchasedOn);
    const formattedDate = originalDate.toISOString().replace(/T/, ' ').replace(/\..+/, '');

    const productNames = productNameList[index] || []; // Get product names for this order

    return (
      <tr key={index}>
        <td>{formattedDate}</td>
        <td>
          <ul style={{ listStyleType: 'none', padding: 0 }}>
            {productNames.map((name, productIndex) => (
              <li key={productIndex} className="text-start">{name}</li>
            ))}
          </ul>
        </td>
        <td>
          <ul style={{ listStyleType: 'none', padding: 0 }}>
            {order.products.map((product, productIndex) => (
              <li key={productIndex}>{product.quantity}</li>
            ))}
          </ul>
        </td>
        <td>
          <ul style={{ listStyleType: 'none', padding: 0 }}>
            {order.products.map((product, productIndex) => (
              <li key={productIndex}>{product.price.toLocaleString('en-PH', { style: 'currency', currency: 'PHP' })}</li>
            ))}
          </ul>
        </td>
        <td>{totalAmount.toLocaleString('en-PH', { style: 'currency', currency: 'PHP' })}</td>
      </tr>
    );
  });

  return (
    <div className="container p-5 forms d-flex flex-column align-items-center">
      <h2 className="mb-5 headers">User Orders</h2>
      <div className="container-fluid">
        {isLoading ? ( // Show spinner if loading
          <Spinner animation="border" role="status">
            <span className="visually-hidden">Loading...</span>
          </Spinner>
        ) : (
          <Table striped bordered hover responsive className="h5 text-center">
            <thead className="text-center">
              <tr>
                <th className="h3">Purchase Date</th>
                <th className="h3">Product</th>
                <th className="h3">Order Quantity</th>
                <th className="h3">Price</th>
                <th className="h3">Total Amount</th>
              </tr>
            </thead>
            <tbody>
              {OrderList}
            </tbody>
          </Table>
        )}
      </div>
    </div>
  );
};

export default ViewUserOrders;
